module Gdas
  module Clients
    # This class extends the cht-lib-fabric redis client to override the call_redis_with_retry method
    # to catch a Redis::ConnectionError, which we frequently see at startup time.
    # This also uses our registered logger, rather than @redis_client.logger, which is broken.
    #
    # NOTE: We are using an old version of cht-lib-fabric (5.3.1) and cannot update to
    # the latest until we have updated the JRuby version for cht-svc-gdas.
    class Redis < ::Platform::Clients::Redis

      # multi does not seem to work with call_redis_with_retry. This overrides
      # the client in cht-lib-fabric.
      def multi(&block)
        @client.send(:multi, &block)
      end

      def call_redis_with_retry(func, *args)
        attempts = 0
        begin
          @client.send(func, *args)
        rescue ::Redis::TimeoutError, ::Redis::CommandError, ::Redis::CannotConnectError, ::Redis::ConnectionError,
          ::Errno::ETIMEDOUT, ::Errno::ECONNREFUSED => ex
          raise if ex.is_a?(::Redis::CommandError) && ex.message != 'LOADING Redis is loading the dataset in memory'
          attempts += 1
          ImportRegistry.resolve('logger').error "#{ex.class}: #{ex.message} (Attempt: #{attempts})"
          if attempts <= 10
            sleep 0.2 * attempts
            retry
          end
          raise
        rescue => ex
          if RUBY_PLATFORM == 'java' && ex.message.blank?
            attempts += 1
            if attempts <= 10
              sleep 0.2 * attempts
              retry
            end
            raise "Empty exception thrown by Redis (Original class: #{ex.class}, Inspect: #{ex.inspect}, func: #{func})"
          end
          raise
        end
      end
    end
  end
end
