# This file contains elements of cht_shared/lib/cht_shared/ec2_os.rb
module Gdas::Constants
  module Ec2Os
    extend ::ActiveSupport::Concern

    included do
      LINUX         ||= :linux
      RHEL          ||= :rhel
      RHEL_HA       ||= :rhelHA
      RHEL_SQL_STD  ||= :rhelSQL
      RHEL_SQL_ENT  ||= :rhelSQLEnterprise
      RHEL_SQL_SW   ||= :rhelSQLWeb
      SLES          ||= :sles
      UBUNTU_PRO    ||= :ubuntuPro
      MSWIN         ||= :mswin
      MSWIN_BYOL    ||= :mswinBYOL
      MSWIN_SQL_WEB ||= :mswinSQLWeb
      MSWIN_SQL_STD ||= :mswinSQL
      MSWIN_SQL_ENT ||= :mswinSQLEnterprise
      LINUX_SQL_WEB ||= :linuxSQLWeb
      LINUX_SQL_STD ||= :linuxSQL
      LINUX_SQL_ENT ||= :linuxSQLEnterprise

      ALL_EC2_OS ||= [LINUX, MSWIN, MSWIN_BYOL, RHEL, RHEL_HA, RHEL_SQL_STD, RHEL_SQL_ENT, RHEL_SQL_SW, SLES,
                      MSWIN_SQL_STD, MSWIN_SQL_WEB, MSWIN_SQL_ENT,
                      LINUX_SQL_STD, LINUX_SQL_WEB, LINUX_SQL_ENT, UBUNTU_PRO].freeze


      # The order of the keys is important as it'll determine the matching order.
      # More verbose keys should always precede less verbose keys to ensure correct matching.
      PRODUCT_TO_OS_MAP = {
        'Red Hat Enterprise Linux with HA'   => RHEL_HA,
        'Red Hat Enterprise Linux'           => RHEL,
        'RHEL with SQL Standard'             => RHEL_SQL_STD,
        'RHEL with SQL Server Enterprise'    => RHEL_SQL_ENT,
        'RHEL with SQL Web'                  => RHEL_SQL_SW,
        'SUSE Linux'                         => SLES,
        'Windows with SQL Server Standard'   => MSWIN_SQL_STD,
        'Windows with SQL Server Web'        => MSWIN_SQL_WEB,
        'Windows with SQL Server Enterprise' => MSWIN_SQL_ENT,
        # If we see SQL server but no edition, assume it's WEB edition
        'Windows with SQL Server'            => MSWIN_SQL_WEB,
        'Windows BYOL'                       => MSWIN_BYOL,
        'Windows'                            => MSWIN,
        'Linux with SQL Server Standard'     => LINUX_SQL_STD,
        'Linux with SQL Server Web'          => LINUX_SQL_WEB,
        'Linux with SQL Server Enterprise'   => LINUX_SQL_ENT,
        # If we see SQL server but no edition, assume it's WEB edition
        'Linux with SQL Server'              => LINUX_SQL_WEB,
        'Linux/UNIX'                         => LINUX,
        'Ubuntu Pro'                         => UBUNTU_PRO
      }.freeze

      # E.g., for Linux, matching operations can be:
      #   RunInstances
      #   RunInstances:S0012
      #   RunInstances:SV006
      # For rest of the OS's, operations match the exact string.
      OPERATION_TO_OS_MAP = {
        'RunInstances:0002'       => MSWIN.to_s,
        'RunInstances:0800'       => MSWIN_BYOL.to_s,
        'RunInstances:0006'       => MSWIN_SQL_STD.to_s,
        'RunInstances:0202'       => MSWIN_SQL_WEB.to_s,
        'RunInstances:0102'       => MSWIN_SQL_ENT.to_s,
        'RunInstances:000g'       => SLES.to_s,
        'RunInstances:0g00'       => UBUNTU_PRO.to_s,
        'RunInstances:0010'       => RHEL.to_s,
        'RunInstances:1010'       => RHEL_HA.to_s,
        'RunInstances:0014'       => RHEL_SQL_STD.to_s,
        'RunInstances:0110'       => RHEL_SQL_ENT.to_s,
        'RunInstances:0210'       => RHEL_SQL_SW.to_s,
        'RunInstances(:SV?\d+)?$' => LINUX.to_s,
        'RunInstances:0004'       => LINUX_SQL_STD.to_s,
        'RunInstances:0200'       => LINUX_SQL_WEB.to_s,
        'RunInstances:0100'       => LINUX_SQL_ENT.to_s
      }

      RE_OPERATION_TO_OS_MAP = OPERATION_TO_OS_MAP.inject({}) do |h, v|
        # Replace string key with compiled Regexp for lookup
        h[Regexp.new(v[0])] = v[1]
        h
      end
    end
  end
end
